@@ -2,7 +2,7 @@
  *
  *  This file provides a means of configuring the plugin.
  *
- *  Copyright (c) 2007 Diego Ongaro <ongardie@gmail.com>
+ *  Copyright (c) 2007-2008 Diego Ongaro <ongardie@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,44 +27,13 @@
 
 #include <gtk/gtk.h>
 
-#include <libxfce4panel/xfce-panel-plugin.h>
-#include <libxfce4panel/xfce-panel-convenience.h>
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4util/libxfce4util.h>
+#include <libxfce4panel/libxfce4panel.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include "cfg.h"
 #include "view.h"
 
-/********** PlacesCfgViewIface helpers **********/
-
-inline PlacesCfg*
-places_cfg_view_iface_get_cfg(PlacesCfgViewIface *iface)
-{
-    return iface->cfg;
-}
-
-inline void
-places_cfg_view_iface_open_dialog(PlacesCfgViewIface *iface)
-{
-    iface->open_dialog(iface->cfg);
-}
-
-inline void
-places_cfg_view_iface_load(PlacesCfgViewIface *iface)
-{
-    iface->load(iface->cfg);
-}
-
-inline void
-places_cfg_view_iface_save(PlacesCfgViewIface *iface)
-{
-    iface->save(iface->cfg);
-}
-
-inline void
-places_cfg_view_iface_finalize(PlacesCfgViewIface *iface)
-{
-    iface->finalize(iface);
-}
 
 
 /********** Configuration File **********/
@@ -77,6 +46,7 @@ pcfg_init_defaults(PlacesCfg *cfg)
     cfg->show_button_label  = FALSE;
     cfg->show_icons         = TRUE;
     cfg->show_volumes       = TRUE;
+    cfg->mount_open_volumes = FALSE;
     cfg->show_bookmarks     = TRUE;
 #if USE_RECENT_DOCUMENTS
     cfg->show_recent        = TRUE;
@@ -121,14 +91,16 @@ pcfg_load(PlacesCfg *cfg)
 
     cfg->show_icons = xfce_rc_read_bool_entry(rcfile, "show_icons", TRUE);
 
-    cfg->show_volumes   = xfce_rc_read_bool_entry(rcfile, "show_volumes", TRUE);
+    cfg->show_volumes       = xfce_rc_read_bool_entry(rcfile, "show_volumes", TRUE);
+    cfg->mount_open_volumes = xfce_rc_read_bool_entry(rcfile, "mount_open_volumes", FALSE);
+
     cfg->show_bookmarks = xfce_rc_read_bool_entry(rcfile, "show_bookmarks", TRUE);
 
     if(cfg->label != NULL)
         g_free(cfg->label);
 
     cfg->label = (gchar*) xfce_rc_read_entry(rcfile, "label", NULL);
-    if(cfg->label == NULL || strlen(cfg->label) == 0)
+    if(cfg->label == NULL || *cfg->label == '\0')
         cfg->label = _("Places");
     cfg->label = g_strdup(cfg->label);
 
@@ -150,11 +122,11 @@ pcfg_load(PlacesCfg *cfg)
     xfce_rc_close(rcfile);
 }
 
-static void
-pcfg_save(PlacesCfg *cfg)
+void
+places_cfg_save(PlacesCfg *cfg)
 {
     XfceRc *rcfile;
-    
+
     g_assert(cfg != NULL);
 
     if(cfg->write_path == NULL)
@@ -172,6 +144,7 @@ pcfg_save(PlacesCfg *cfg)
     /* MENU */
     xfce_rc_write_bool_entry(rcfile, "show_icons", cfg->show_icons);
     xfce_rc_write_bool_entry(rcfile, "show_volumes", cfg->show_volumes);
+    xfce_rc_write_bool_entry(rcfile, "mount_open_volumes", cfg->mount_open_volumes);
     xfce_rc_write_bool_entry(rcfile, "show_bookmarks", cfg->show_bookmarks);
 #if USE_RECENT_DOCUMENTS
     xfce_rc_write_bool_entry(rcfile, "show_recent", cfg->show_recent);
@@ -196,7 +169,7 @@ static void
 pcfg_button_show_cb(GtkComboBox *combo, PlacesCfg *cfg)
 {
     gint option = gtk_combo_box_get_active(combo);
-    
+
     g_assert(cfg != NULL);
     g_assert(option >= 0 && option <= 2);
 
@@ -217,12 +190,12 @@ pcfg_button_label_cb(GtkWidget *label_entry, GdkEventFocus *event, PlacesCfg *cf
 
     entry_text = gtk_entry_get_text(GTK_ENTRY(label_entry));
     new_text = g_strstrip(g_strdup(entry_text));
-    if(old_text == NULL || (strcmp(old_text, new_text) && strlen(new_text))){
+    if(old_text == NULL || (strcmp(old_text, new_text) && *new_text != '\0')){
         cfg->label = new_text;
 
         if(old_text != NULL)
             g_free(old_text);
-    
+
         places_view_cfg_iface_update_button(cfg->view_iface);
 
     }else{ /* we prefer the old/default text */
@@ -230,7 +203,7 @@ pcfg_button_label_cb(GtkWidget *label_entry, GdkEventFocus *event, PlacesCfg *cf
         if(old_text == NULL)
             cfg->label = g_strdup(_("Places"));
 
-        if(old_text == NULL || !strlen(new_text)){
+        if(old_text == NULL || *new_text == '\0'){
             gtk_entry_set_text(GTK_ENTRY(label_entry), cfg->label);
             places_view_cfg_iface_update_button(cfg->view_iface);
         }
@@ -255,10 +228,10 @@ pcfg_search_cmd_cb(GtkWidget *label_entry, GdkEventFocus *event, PlacesCfg *cfg)
 
     if(old_text == NULL || strcmp(old_text, new_text)){
         cfg->search_cmd = new_text;
-        
+
         if(old_text != NULL)
             g_free(old_text);
-        
+
         places_view_cfg_iface_update_menu(cfg->view_iface);
 
     }else /* we prefer the old text */
@@ -284,6 +257,8 @@ static void
 pcfg_menu_cb(GtkToggleButton *toggle, PlacesCfg *cfg)
 {
     gboolean *opt;
+    GtkWidget *transient;
+
     g_assert(cfg != NULL);
 
     opt = g_object_get_data(G_OBJECT(toggle), "cfg_opt");
@@ -291,6 +266,10 @@ pcfg_menu_cb(GtkToggleButton *toggle, PlacesCfg *cfg)
 
     *opt = gtk_toggle_button_get_active(toggle);
 
+    transient = g_object_get_data(G_OBJECT(toggle), "cfg_transient");
+    if(transient != NULL)
+        gtk_widget_set_sensitive(transient, *opt);
+
     places_view_cfg_iface_update_menu(cfg->view_iface);
 }
 
@@ -298,21 +277,53 @@ static void
 pcfg_model_cb(GtkToggleButton *toggle, PlacesCfg *cfg)
 {
     gboolean *opt;
+    GtkWidget *transient;
+
     g_assert(cfg != NULL);
-    
+
     opt = g_object_get_data(G_OBJECT(toggle), "cfg_opt");
     g_assert(opt != NULL);
 
     *opt = gtk_toggle_button_get_active(toggle);
 
+    transient = g_object_get_data(G_OBJECT(toggle), "cfg_transient");
+    if(transient != NULL)
+        gtk_widget_set_sensitive(transient, *opt);
+
     places_view_cfg_iface_reconfigure_model(cfg->view_iface);
 }
 
 static void
-pcfg_open_dialog(PlacesCfg *cfg)
+pcfg_dialog_close_cb(GtkDialog *dialog, gint response, PlacesCfg *cfg)
 {
-    DBG("configure plugin");
+    gtk_widget_destroy(GTK_WIDGET(dialog));
+    xfce_panel_plugin_unblock_menu(cfg->plugin);
+}
 
+static GtkWidget*
+pcfg_make_empty_dialog(PlacesCfg *cfg)
+{
+    GtkWidget *dlg; /* we'll return this */
+
+    xfce_panel_plugin_block_menu(cfg->plugin);
+
+    dlg = xfce_titled_dialog_new_with_buttons(_("Places"),
+              NULL,
+              GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
+              GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
+
+    gtk_window_set_position(GTK_WINDOW(dlg), GTK_WIN_POS_CENTER);
+    gtk_window_set_icon_name(GTK_WINDOW(dlg), "xfce4-settings");
+
+    g_signal_connect(dlg, "response",
+                     G_CALLBACK(pcfg_dialog_close_cb), cfg);
+
+    return dlg;
+}
+
+void
+places_cfg_open_dialog(PlacesCfg *cfg)
+{
     GtkWidget *dlg;
     GtkWidget *frame_button, *vbox_button;
     GtkWidget *frame_menu,   *vbox_menu;
@@ -322,15 +333,18 @@ pcfg_open_dialog(PlacesCfg *cfg)
     GtkWidget *frame_search, *vbox_search;
 
     GtkWidget *tmp_box, *tmp_label, *tmp_widget;
+    GtkObject * adj;
     gint active;
-    
-    dlg = places_view_cfg_iface_make_empty_cfg_dialog(cfg->view_iface);
+
+    DBG("configure plugin");
+
+    dlg = pcfg_make_empty_dialog(cfg);
 
     /* BUTTON: frame, vbox */
     vbox_button = gtk_vbox_new(FALSE, 4);
     gtk_widget_show(vbox_button);
-    
-    frame_button = xfce_create_framebox_with_content(_("Button"), vbox_button);
+
+    frame_button = xfce_gtk_frame_box_new_with_content(_("Button"), vbox_button);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), frame_button, FALSE, FALSE, 0);
 
 
@@ -338,7 +352,7 @@ pcfg_open_dialog(PlacesCfg *cfg)
     tmp_box = gtk_hbox_new(FALSE, 15);
     gtk_widget_show(tmp_box);
     gtk_box_pack_start(GTK_BOX(vbox_button), tmp_box, FALSE, FALSE, 0);
-    
+
     tmp_label = gtk_label_new_with_mnemonic(_("_Show"));
     gtk_widget_show(tmp_label);
     gtk_box_pack_start(GTK_BOX(tmp_box), tmp_label, FALSE, FALSE, 0);
@@ -357,7 +371,7 @@ pcfg_open_dialog(PlacesCfg *cfg)
     else
         active = 0;
     gtk_combo_box_set_active(GTK_COMBO_BOX(tmp_widget), active);
-    
+
     g_signal_connect(G_OBJECT(tmp_widget), "changed",
                      G_CALLBACK(pcfg_button_show_cb), cfg);
 
@@ -368,7 +382,7 @@ pcfg_open_dialog(PlacesCfg *cfg)
     tmp_box = gtk_hbox_new(FALSE, 15);
     gtk_widget_show(tmp_box);
     gtk_box_pack_start(GTK_BOX(vbox_button), tmp_box, FALSE, FALSE, 0);
-    
+
     tmp_label = gtk_label_new_with_mnemonic(_("_Label"));
     gtk_widget_show(tmp_label);
     gtk_box_pack_start(GTK_BOX(tmp_box), tmp_label, FALSE, FALSE, 0);
@@ -386,8 +400,8 @@ pcfg_open_dialog(PlacesCfg *cfg)
     /* MENU: frame, vbox */
     vbox_menu = gtk_vbox_new(FALSE, 4);
     gtk_widget_show(vbox_menu);
-    
-    frame_menu = xfce_create_framebox_with_content(_("Menu"), vbox_menu);
+
+    frame_menu = xfce_gtk_frame_box_new_with_content(_("Menu"), vbox_menu);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), frame_menu, FALSE, FALSE, 0);
 
     /* MENU: Show Icons */
@@ -411,7 +425,31 @@ pcfg_open_dialog(PlacesCfg *cfg)
                      G_CALLBACK(pcfg_model_cb), cfg);
 
     gtk_widget_show(tmp_widget);
-    gtk_box_pack_start(GTK_BOX(vbox_menu), tmp_widget, FALSE, FALSE, 0);   
+    gtk_box_pack_start(GTK_BOX(vbox_menu), tmp_widget, FALSE, FALSE, 0);
+
+    /* MENU: - Mount and Open (indented) */
+    tmp_box = gtk_hbox_new(FALSE, 15);
+
+    /* Gray out this box when "Show removable media" is off */
+    gtk_widget_set_sensitive(tmp_box, cfg->show_volumes);
+    g_object_set_data(G_OBJECT(tmp_widget), "cfg_transient", tmp_box);
+
+    tmp_widget = gtk_label_new(" "); /* TODO: is there a more appropriate widget? */
+    gtk_widget_show(tmp_widget);
+    gtk_box_pack_start(GTK_BOX(tmp_box), tmp_widget, FALSE, FALSE, 0);
+
+    tmp_widget = gtk_check_button_new_with_mnemonic(_("Mount and _Open on click"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget), cfg->mount_open_volumes);
+
+    g_object_set_data(G_OBJECT(tmp_widget), "cfg_opt", &(cfg->mount_open_volumes));
+    g_signal_connect(G_OBJECT(tmp_widget), "toggled",
+                     G_CALLBACK(pcfg_model_cb), cfg);
+
+    gtk_widget_show(tmp_widget);
+    gtk_box_pack_start(GTK_BOX(tmp_box), tmp_widget, FALSE, FALSE, 0);
+
+    gtk_widget_show(tmp_box);
+    gtk_box_pack_start(GTK_BOX(vbox_menu), tmp_box, FALSE, FALSE, 0);
 
     /* MENU: Show GTK Bookmarks */
     tmp_widget = gtk_check_button_new_with_mnemonic(_("Show GTK _bookmarks"));
@@ -440,8 +478,12 @@ pcfg_open_dialog(PlacesCfg *cfg)
     /* RECENT DOCUMENTS: frame, vbox */
     vbox_recent = gtk_vbox_new(FALSE, 4);
     gtk_widget_show(vbox_recent);
-    
-    frame_recent = xfce_create_framebox_with_content(_("Recent Documents"), vbox_recent);
+
+    /* Gray out this box when "Show recent documents" is off */
+    gtk_widget_set_sensitive(vbox_recent, cfg->show_recent);
+    g_object_set_data(G_OBJECT(tmp_widget), "cfg_transient", vbox_recent);
+
+    frame_recent = xfce_gtk_frame_box_new_with_content(_("Recent Documents"), vbox_recent);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), frame_recent, FALSE, FALSE, 0);
 
     /* RECENT DOCUMENTS: Show clear option */
@@ -459,12 +501,12 @@ pcfg_open_dialog(PlacesCfg *cfg)
     tmp_box = gtk_hbox_new(FALSE, 15);
     gtk_widget_show(tmp_box);
     gtk_box_pack_start(GTK_BOX(vbox_recent), tmp_box, FALSE, FALSE, 0);
-    
+
     tmp_label = gtk_label_new_with_mnemonic(_("_Number to display"));
     gtk_widget_show(tmp_label);
     gtk_box_pack_start(GTK_BOX(tmp_box), tmp_label, FALSE, FALSE, 0);
 
-    GtkObject *adj = gtk_adjustment_new(cfg->show_recent_number, 1, 25, 1, 5, 5);
+    adj = gtk_adjustment_new(cfg->show_recent_number, 1, 25, 1, 5, 5);
 
     tmp_widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
     gtk_label_set_mnemonic_widget(GTK_LABEL(tmp_label), tmp_widget);
@@ -479,15 +521,15 @@ pcfg_open_dialog(PlacesCfg *cfg)
     /* Search: frame, vbox */
     vbox_search = gtk_vbox_new(FALSE, 4);
     gtk_widget_show(vbox_search);
-    
-    frame_search = xfce_create_framebox_with_content(_("Search"), vbox_search);
+
+    frame_search = xfce_gtk_frame_box_new_with_content(_("Search"), vbox_search);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), frame_search, FALSE, FALSE, 0);
 
     /* Search: command */
     tmp_box = gtk_hbox_new(FALSE, 15);
     gtk_widget_show(tmp_box);
     gtk_box_pack_start(GTK_BOX(vbox_search), tmp_box, FALSE, FALSE, 0);
-    
+
     tmp_label = gtk_label_new_with_mnemonic(_("Co_mmand"));
     gtk_widget_show(tmp_label);
     gtk_box_pack_start(GTK_BOX(tmp_box), tmp_label, FALSE, FALSE, 0);
@@ -502,19 +544,14 @@ pcfg_open_dialog(PlacesCfg *cfg)
     gtk_widget_show(tmp_widget);
     gtk_box_pack_start(GTK_BOX(tmp_box), tmp_widget, FALSE, FALSE, 0);
 
-    gtk_widget_show(dlg);
+    gtk_widget_show_all(dlg);
 }
 
 /********** Initialization & Finalization **********/
 
-static void 
-pcfg_finalize(PlacesCfgViewIface *cfg_iface)
+void
+places_cfg_finalize(PlacesCfg *cfg)
 {
-    PlacesCfg *cfg;
-
-    g_assert(cfg_iface != NULL);
-
-    cfg = cfg_iface->cfg;
     g_assert(cfg != NULL);
 
     if(cfg->label != NULL)
@@ -528,32 +565,31 @@ pcfg_finalize(PlacesCfgViewIface *cfg_iface)
         g_free(cfg->write_path);
 
     g_free(cfg);
-    g_free(cfg_iface);
 }
 
-/* cfg takes ownership of {read,write}_path */
-PlacesCfgViewIface*
-places_cfg_new(PlacesViewCfgIface *view_iface, gchar *read_path, gchar *write_path)
+PlacesCfg*
+places_cfg_new(XfcePanelPlugin *plugin, PlacesViewCfgIface *view_iface)
 {
-    PlacesCfgViewIface *cfg_iface;
     PlacesCfg *cfg;
 
     g_assert(view_iface != NULL);
 
-    cfg                     = g_new0(PlacesCfg, 1);
-    cfg->read_path          = read_path;
-    cfg->write_path         = write_path;
-    cfg->view_iface         = view_iface;
+    cfg             = g_new0(PlacesCfg, 1);
+    cfg->plugin     = plugin;
+    cfg->view_iface = view_iface;
+
+    cfg->read_path  = xfce_panel_plugin_lookup_rc_file(plugin);
+    cfg->write_path = xfce_panel_plugin_save_location(plugin, TRUE);
 
     pcfg_load(cfg);
-    
-    cfg_iface               = g_new0(PlacesCfgViewIface, 1);
-    cfg_iface->cfg          = cfg;
-    cfg_iface->open_dialog  = pcfg_open_dialog;
-    cfg_iface->save         = pcfg_save;
-    cfg_iface->finalize     = pcfg_finalize;
 
-    return cfg_iface;
+    g_signal_connect_swapped(G_OBJECT(plugin), "configure-plugin",
+                             G_CALLBACK(places_cfg_open_dialog), cfg);
+    g_signal_connect_swapped(G_OBJECT(plugin), "save",
+                             G_CALLBACK(places_cfg_save), cfg);
+    xfce_panel_plugin_menu_show_configure(plugin);
+
+    return cfg;
 }
 
 /* vim: set ai et tabstop=4: */
