@@ -23,7 +23,7 @@
 #define _XFCE_PANEL_PLACES_CFG_H
 
 #include <glib.h>
-#include <libxfce4panel/xfce-panel-plugin.h>
+#include <libxfce4panel/libxfce4panel.h>
 #include "view.h"
 
 typedef struct
