@@ -49,18 +49,14 @@
 #define USE_GTK_TOOLTIP_API     GTK_CHECK_VERSION(2,12,0)
 
 #include <libxfce4util/libxfce4util.h>
-#include <libxfce4panel/xfce-panel-plugin.h>
-#include <libxfce4panel/xfce-panel-convenience.h>
-#include <libxfce4panel/xfce-hvbox.h>
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4panel/libxfce4panel.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #define EXO_API_SUBJECT_TO_CHANGE
 #include <exo/exo.h>
 
 #include <string.h>
 
-#include "xfce46-compat.h"
-
 #include "view.h"
 #include "support.h"
 #include "cfg.h"
@@ -71,6 +67,10 @@
 #include "xfce4-popup-places.h"
 #include "button.h"
 
+#ifdef HAVE_LIBNOTIFY
+#include "model_volumes_notify.h"
+#endif
+
 struct _PlacesViewCfgIface
 {
     PlacesView          *places_view;
@@ -456,6 +456,42 @@ pview_cb_recent_items_clear3(GtkWidget *clear_item, GdkEventButton *event, GtkWi
 
 /********** UI Helpers **********/
 
+static GdkPixbuf *
+pview_get_icon(GIcon *icon)
+{
+    GtkIconTheme *itheme = gtk_icon_theme_get_default();
+    GdkPixbuf *pix = NULL;
+    gint width, height, size;
+
+    g_return_val_if_fail(icon != NULL, NULL);
+
+    if (gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height))
+        size = MAX(width, height);
+    else
+        size = 32;
+
+    if (G_IS_THEMED_ICON(icon)) {
+        GtkIconInfo *icon_info = gtk_icon_theme_lookup_by_gicon(itheme,
+                                                                icon, size,
+                                                                GTK_ICON_LOOKUP_USE_BUILTIN | GTK_ICON_LOOKUP_GENERIC_FALLBACK | GTK_ICON_LOOKUP_FORCE_SIZE);
+        if (icon_info) {
+            GdkPixbuf *pix_theme = gtk_icon_info_load_icon(icon_info, NULL);
+            pix = gdk_pixbuf_copy(pix_theme);
+            gtk_icon_info_free(icon_info);
+            g_object_unref(G_OBJECT(pix_theme));
+        }
+    } else if(G_IS_LOADABLE_ICON(icon)) {
+        GInputStream *stream = g_loadable_icon_load(G_LOADABLE_ICON(icon),
+                                                    size, NULL, NULL, NULL);
+        if (stream) {
+            pix = gdk_pixbuf_new_from_stream(stream, NULL, NULL);
+            g_object_unref(stream);
+        }
+    }
+
+    return pix;
+}
+
 static void
 pview_destroy_menu(PlacesView *view)
 {
@@ -483,14 +519,14 @@ pview_destroy_menu(PlacesView *view)
 static void
 pview_add_menu_item(PlacesView *view, PlacesBookmark *bookmark)
 {
-    g_assert(view != NULL);
-    g_assert(bookmark != NULL);
-
     GtkWidget *separator;
     GtkWidget *item;
     GdkPixbuf *pb;
     GtkWidget *image;
 
+    g_assert(view != NULL);
+    g_assert(bookmark != NULL);
+
     /* lazily add separator */
     if(view->needs_separator){
 
@@ -505,13 +541,7 @@ pview_add_menu_item(PlacesView *view, PlacesBookmark *bookmark)
 
     /* try to set icon */
     if(view->cfg->show_icons && bookmark->icon != NULL){
-        gint icon_size;
-        gint width, height;
-        if (gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height))
-                icon_size = MAX(width, height);
-        else
-                icon_size = 32;
-        pb = xfce_themed_icon_load(bookmark->icon, icon_size);
+        pb = pview_get_icon(bookmark->icon);
 
         if(G_LIKELY(pb != NULL)){
             image = gtk_image_new_from_pixbuf(pb);
@@ -739,7 +769,19 @@ pview_open_menu(PlacesView *pd)
 static GdkPixbuf*
 pview_pixbuf_factory(gint size)
 {
-    return xfce_themed_icon_load_category(2, size);
+   static const gchar *icons[] = { "system-file-manager",
+                                   "xfce-filemanager",
+                                   "file-manager",
+                                   "folder",
+                                   NULL };
+   int i = 0;
+   GdkPixbuf *pb = NULL;
+
+   while (icons[i] && !pb) {
+      pb = xfce_panel_pixbuf_from_source(icons[i], NULL, size);
+      i++;
+   }
+   return pb;
 }
 
 static void
@@ -912,6 +954,10 @@ places_view_finalize(PlacesView *view)
     view->view_cfg_iface = NULL;
 
     g_free(view);
+
+#ifdef HAVE_LIBNOTIFY
+    pbvol_notify_uninit();
+#endif
 }
 
 /* vim: set ai et tabstop=4: */
